// based on Olivier Pelhatre's GTK 3 example, ported to GTK 4
//
//   https://github.com/Ouam74/RUST_Real-time_plots_using_GTK-rs_and_Plotters-rs
//
// a self-contained component might draw on the example below, by StackOverflow
// user Nicolas
//
//   https://stackoverflow.com/a/76548487
//
// here's a crash course in `plotters`
//
//   https://plotters-rs.github.io/book/basic/basic_data_plotting.html
//

extern crate cairo;
use plotters::prelude::*;
use plotters_cairo::CairoBackend;
use gtk::{
    glib,
    prelude::*,
    Adjustment,
    Align,
    Application,
    ApplicationWindow,
    Box,
    DrawingArea,
    Label,
    Orientation,
    Scale
};
use std::time::Instant;

mod engine;

fn main() -> glib::ExitCode {
    let app = Application::builder()
        .application_id("org.studioinfinity.rust-benchmark-native")
        .build();
    
    app.connect_activate(|app| {
        const TIME_RES: usize = 100;
        let start_time = Instant::now();
        let eigval_series = engine::rand_eigval_series(60, TIME_RES);
        let run_time = start_time.elapsed().as_millis();
        
        // application state
        let time_step = Adjustment::new(0.0, 0.0, TIME_RES as f64, 1.0, 0.0, 0.0);
        
        // create the window.
        let window = ApplicationWindow::builder()
            .application(app)
            .title("The circular law")
            .build();
        
        // create a vertical box
        let container = Box::new(Orientation::Vertical, 5);
        window.set_child(Some(&container));
        
        // create the run time readout
        let run_time_readout = Label::builder()
            .margin_top(5)
            .margin_start(10)
            .halign(Align::Start)
            .label(glib::gformat!("{} ms", run_time))
            .build();
        container.append(&run_time_readout);
        
        // set up the drawing area
        let drawing_area = DrawingArea::builder()
            .content_width(600)
            .content_height(600)
            .build();
        let time_step_for_draw = time_step.clone();
        let draw_eigvals = move |_: &DrawingArea, context: &cairo::Context, width: i32, height: i32| {
            let root = CairoBackend::new(&context, (width as u32, height as u32)).unwrap().into_drawing_area();
            let _ = root.fill(&BLACK);
            
            const R_DISP: f64 = 1.5;
            let mut chart = ChartBuilder::on(&root)
                .build_cartesian_2d(-R_DISP..R_DISP, -R_DISP..R_DISP)
                .unwrap();
            let time_step_val = (time_step_for_draw.value() as usize).min(TIME_RES-1);
            let eigval_iter = eigval_series[time_step_val].iter();
            let _ = chart.draw_series(
                eigval_iter.map(|z| Circle::new((z.re, z.im), 3, WHITE.filled()))
            );
            let _ = root.present();
        };
        DrawingAreaExtManual::set_draw_func(&drawing_area, draw_eigvals);
        container.append(&drawing_area);
        
        // set up the time step slider
        let time_step_scale = Scale::new(Orientation::Horizontal, Some(&time_step));
        time_step_scale.connect_value_changed(move |_: &Scale| {
            drawing_area.queue_draw();
        });
        container.append(&time_step_scale);
        
        // show the window
        window.present();
    });
    
    app.run()
}